/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include "DesQShell.hpp"
#include "DesQShellAdaptor.hpp"
#include "DesQAutoStart.hpp"
#include "BrightnessWidget.hpp"
#include "ClockWidget.hpp"
#include "ResourcesWidget.hpp"
#include "VolumeWidget.hpp"

#include "DesQMenu.hpp"

#include <desq/DesQUtils.hpp>

// Wayland Headers
#include <desqwl/DesQWlGlobal.hpp>
#include <desqwl/AutoHideWindow.hpp>
#include <desqwl/Registry.hpp>
#include <desqwl/LayerShell.hpp>

/* Qt Headers */
#include <QtDBus>
#include <QLocalSocket>

QLocalSocket *socket;

static inline void updateStatus( QString status ) {

	socket->connectToServer( "org.DesQ.Shell.Splash" );
	socket->write( status.toUtf8() );
	socket->flush();

	if ( status != "quit" )
		qDebug() << status;
};

DesQShell::DesQShell( QString imageLoc ) {

	socket = new QLocalSocket();

	updateStatus( "Preparing to Setup DesQ..." );
	/* We need our desktop to fill the whole physical screen */
	setFixedSize( qApp->primaryScreen()->size() );

	/* If the display manager is X11, we need to let it know that we are a desktop */
	setAttribute( Qt::WA_X11NetWmWindowTypeDesktop );

	/* Title */
	setWindowTitle( "DesQ Shell" );

	/* No frame, and stay at bottom */
	Qt::WindowFlags wFlags = Qt::FramelessWindowHint | Qt::WindowStaysOnBottomHint;
	if ( DesQWayland::isRunning() )
		wFlags |= Qt::BypassWindowManagerHint;

	/* Set the windowFlags */
	setWindowFlags( wFlags );

	/* Background image */
	updateStatus( "Reading Settings..." );
	bgImagePath = imageLoc;
    if ( bgImagePath.isEmpty() )
		bgImagePath = ( QString )shellSett->value( "Background" );

	if ( not DesQUtils::exists( bgImagePath ) )
		bgImagePath = "/usr/share/desq/resources/default.jpg";

	/* Connect to the DesQ Session */
	updateStatus( "Connecting to session..." );
	session = new QDBusInterface( "org.DesQ.Session", "/org/DesQ/Session", "org.DesQ.Session", QDBusConnection::sessionBus() );

	/* Prepare the background */
	updateStatus( "Drawing Background..." );
	QTimer::singleShot( 1000, this, &DesQShell::prepareBackgroundImage );

	/* Create the Shell UI */
	updateStatus( "Creating Shell Interface..." );
	createShellInterface();

	/* Create the desktop widgets */
	updateStatus( "Loading Desktop Widgets..." );
	createDesktopWidgets();

	/* Start the apps listed in autostart */
	updateStatus( "Auto-starting Apps..." );
	DesQAutoStart *autoStart = new DesQAutoStart();
	autoStart->start();

	/* Shell DBus Service */
	updateStatus( "Registering Shell Service..." );
	new ShellAdaptor( this );

	QDBusConnection shellDBus = QDBusConnection::sessionBus();
    shellDBus.registerObject( "/org/DesQ/Shell", this );
    shellDBus.registerService( "org.DesQ.Shell" );

	session->call( "RegisterService", "org.DesQ.Shell" );

	/* Close the SplashScreen */
	updateStatus( "quit" );
};

void DesQShell::setupWayland( WaylandRegistry *reg ) {

	registry = reg;

	/*
		*
		*    === HotSpots ===
		* 1. TopLeft - Peek desktop
		* 2. TopRight - Show window list
		* 3. BottomLeft - DesQ Super Button
		* 4. BottomRight - Lock Screen
		*
		* 250 ms relaxation time for the hotspot to be activated
		*
	*/

	qDebug() << (bool)shellSett->value( "ShowPanel" );
	if ( shellSett->value( "ShowPanel" ) ) {
		QProcess::startDetached( DesQUtils::getUtilityPath( "panel" ), {} );
	}

	DesQHotSpot *topLeftHS = new DesQHotSpot( qApp->primaryScreen(), reg->wayfireShell(), this );
	topLeftHS->createHotspot( DesQHotSpot::TopLeft, 1, 250 );
	connect( topLeftHS, &DesQHotSpot::enteredHotspot, [=]() { handleHotCorner( DesQHotSpot::TopLeft ); } );
	connect( topLeftHS, &DesQHotSpot::leftHotspot,    [=]() { cancelHotCorner( DesQHotSpot::TopLeft ); } );

	DesQHotSpot *topRightHS = new DesQHotSpot( qApp->primaryScreen(), reg->wayfireShell(), this );
	topRightHS->createHotspot( DesQHotSpot::TopRight, 1, 250 );
	connect( topRightHS, &DesQHotSpot::enteredHotspot, [=]() { handleHotCorner( DesQHotSpot::TopRight ); } );
	connect( topRightHS, &DesQHotSpot::leftHotspot,    [=]() { cancelHotCorner( DesQHotSpot::TopRight ); } );

	DesQHotSpot *bottomLeftHS = new DesQHotSpot( qApp->primaryScreen(), reg->wayfireShell(), this );
	bottomLeftHS->createHotspot( DesQHotSpot::BottomLeft, 1, 250 );
	connect( bottomLeftHS, &DesQHotSpot::enteredHotspot, [=]() { handleHotCorner( DesQHotSpot::BottomLeft ); } );
	connect( bottomLeftHS, &DesQHotSpot::leftHotspot,    [=]() { cancelHotCorner( DesQHotSpot::BottomLeft ); } );

	DesQHotSpot *bottomRightHS = new DesQHotSpot( qApp->primaryScreen(), reg->wayfireShell(), this );
	bottomRightHS->createHotspot( DesQHotSpot::BottomRight, 1, 250 );
	connect( bottomRightHS, &DesQHotSpot::enteredHotspot, [=]() { handleHotCorner( DesQHotSpot::BottomRight ); } );
	connect( bottomRightHS, &DesQHotSpot::leftHotspot,    [=]() { cancelHotCorner( DesQHotSpot::BottomRight ); } );

	QProcess::startDetached( "pkill", { "desq-splash" } );
};

void DesQShell::prepareBackgroundImage() {

	/* Cache the scaled image */
	cachedImagePath = QDir::home().filePath( ".cache/DesQ/" );
	cachedImagePath += QFileInfo( bgImagePath ).fileTime( QFile::FileModificationTime ).toString( "yyyyMMdd-hhmmss-" );
	cachedImagePath += "background.jpg";

	if ( DesQUtils::exists( cachedImagePath ) ) {
		desktopImage = QImage( cachedImagePath );

		if ( desktopImage.size() == qApp->primaryScreen()->size() )
			return;

		else {
			qDebug() << desktopImage.size() << qApp->primaryScreen()->size();
		}
	}

	/* Currently, the default setting: Scaled and cropped */
	QImage tmpImg = QImage( bgImagePath ).scaled( qApp->primaryScreen()->size(), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation );
	desktopImage = tmpImg.copy( QRect( QPoint( 0, 0 ), qApp->primaryScreen()->size() ) );

	/* Save it for further use */
	desktopImage.save( cachedImagePath );

	repaint();
};

void DesQShell::createShellInterface() {

	/* Start the Status Notifier Watcher */
	QDBusInterface snw( "org.kde.StatusNotifierWatcher", "/StatusNotifierWatcher", "org.kde.StatusNotifierWatcher", QDBusConnection::sessionBus() );
	if ( not snw.isValid() )
		QProcess::startDetached( DesQUtils::getUtilityPath( "snwatcher-daemon" ), { "-d" } );

	qDebug() << "3.1";

	/* A Start Button at bottom right corner */
	startBtn = new QToolButton( this );
	startBtn->setIcon( QIcon( ":/desq.png" ) );
	startBtn->setIconSize( QSize( 64, 64 ) );
	startBtn->setAutoRaise( true );
	startBtn->setFocusPolicy( Qt::NoFocus );
	startBtn->setEnabled( true );
	startBtn->setAttribute( Qt::WA_TranslucentBackground, true );

	/* Application menu */
	shellMenu = new DesQMenu( this );

	qDebug() << "3.2";

	/* Show the menu when clicked */
	connect( startBtn, &QPushButton::clicked, this, &DesQShell::showMenu );
	connect( startBtn, &QPushButton::clicked, [=]() {
		/* In case it's floating */
		if ( not startBtn->parent() ) {
			startBtn->setParent( this );
			startBtn->setWindowFlags( Qt::Widget );
			startBtn->show();
		}
	} );

	connect( qApp->primaryScreen(), &QScreen::geometryChanged, this, &DesQShell::resizeDesktop );

	QVBoxLayout *lyt = new QVBoxLayout();
	lyt->addStretch();
	lyt->addWidget( startBtn );

	desktopLyt = new QGridLayout();
	int panelMargin = ( shellSett->value( "ShowPanel" ) ? shellSett->value( "PanelHeight" ) : 0 );
	desktopLyt->setContentsMargins( QMargins( 0, panelMargin, 0, 0 ) );
	desktopLyt->addLayout( lyt, 0, 0, Qt::AlignLeft );

	setLayout( desktopLyt );

	qDebug() << "3.3";
};

void DesQShell::createDesktopWidgets() {

	QVBoxLayout *widgetsLyt = new QVBoxLayout();

	ClockWidget *cw = new ClockWidget( this );
	cw->setFixedSize( 200, 200 );
	qDebug() << "4.1";

	ResourcesWidget *res = new ResourcesWidget( this );
	res->setFixedSize( 200, 200 );
	qDebug() << "4.2";

	BrightnessWidget *display = new BrightnessWidget( this );
	display->setFixedSize( 200, 120 );
	qDebug() << "4.3";

	VolumeWidget *volume = new VolumeWidget( this );
	volume->setFixedSize( 200, 120 );
	qDebug() << "4.4";

	widgetsLyt->addWidget( cw );

	widgetsLyt->addWidget( display );
	widgetsLyt->addWidget( volume );
	widgetsLyt->addWidget( res );
	widgetsLyt->addStretch();

	desktopLyt->addLayout( widgetsLyt, 0, 1, Qt::AlignRight );
	qDebug() << "4.5";
};

void DesQShell::handleHotCorner( int corner ) {

	/* Based on the corner take action */
	switch( corner ) {

		case DesQHotSpot::TopLeft: {
			qDebug() << "Entering Top Left hot Corner";
			break;
		}

		case DesQHotSpot::TopRight: {
			qDebug() << "Entering Top Right Corner";
			/* Use TasksPlugin to show the list widget */
			break;
		}

		case DesQHotSpot::BottomLeft: {
			qDebug() << "Entering Bottom Left Corner";
			startBtn->click();
			break;
		}

		case DesQHotSpot::BottomRight: {
			qDebug() << "Entering Bottom Right Corner";
			lockScreen();
			break;
		}

		/* In case hot corner was canceled */
		default: {
			break;
		}
	}
};

void DesQShell::cancelHotCorner( int corner ) {

	switch( corner ) {

		case DesQHotSpot::TopLeft: {
			qDebug() << "Leaving Top Left Corner";
			break;
		}

		/* We are exiting from the TopRight corner, so show restore the window state */
		case DesQHotSpot::TopRight: {
			qDebug() << "Leaving Top Right Corner";
			break;
		}

		case DesQHotSpot::BottomLeft: {
			qDebug() << "Leaving Bottom Left Corner";
			break;
		}

		case DesQHotSpot::BottomRight: {
			qDebug() << "Leaving Bottom Right Corner";
			break;
		}

		/* In case hot corner was canceled */
		default: {
			break;
		}
	}
};

void DesQShell::showMenu() {

	shellMenu->show();
};

void DesQShell::lockScreen() {

	if ( not QProcess::startDetached( "loginctl", { "lock-session" } ) )
		QProcess::startDetached( "swaylock", {} );
};

void DesQShell::resizeDesktop() {

	qDebug() << "Screen resized";
	qDebug() << qApp->primaryScreen()->size();

	/* Reset the background */
	prepareBackgroundImage();

	/* We need our desktop to fill the whole physical screen */
	setFixedSize( qApp->primaryScreen()->size() );
};

void DesQShell::handleMessages( const QString message ) {

	/* Show the drop-down */
	if ( message == "dropdown" ) {
		QProcess::startDetached( "desq-dd", {} );
	}

	/* Change the background image, temporarily */
	else if ( message.startsWith( "background" ) ) {
		bgImagePath = message.split( "\n", Qt::SkipEmptyParts ).takeLast();
		QFile::remove( cachedImagePath );
		prepareBackgroundImage();
	}

	/* Lock the screen */
	else if ( message.startsWith( "lockscreen" ) ) {

		if ( session->isValid() )
			session->call( "lock" );

		else
			lockScreen();
	}
};

void DesQShell::paintEvent( QPaintEvent *pEvent ) {

	if ( not desktopImage.isNull() ) {
		QPainter painter( this );
		painter.drawImage( geometry(), desktopImage );
		painter.end();
	}

	QWidget::paintEvent( pEvent );
};
