/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project ( https://gitlab.com/desq/ )
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include "DesQMenu.hpp"
#include "LogoutDialog.hpp"

// #include <desq/DesQWMSession.hpp>

DesQMenu::DesQMenu( QWidget *parent ) : QWidget( parent ) {

	/* We need our desktop to fill the whole physical screen */
	setFixedSize( qApp->primaryScreen()->size() );

	/* Title */
	setWindowTitle( "DesQ Menu" );

	/* No frame, and stay on top */
    setWindowFlags( Qt::Popup | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint );

    // if ( DesQWMSession::currentSession()->isCompositorRunning() )
	setAttribute( Qt::WA_TranslucentBackground );

	/* Populate the menu */
	createUI();
};

void DesQMenu::createUI() {

	searchEdit = new QLineEdit();
	searchEdit->setPlaceholderText( "Type to search..." );
	searchEdit->setFont( QFont( font().family(), 20 ) );
	searchEdit->setAlignment( Qt::AlignCenter );
	searchEdit->setFrame( QFrame::NoFrame );
	searchEdit->setStyleSheet( "background: transparent; margin-bottom: 10px;" );
	searchEdit->grabKeyboard();
	searchEdit->setMinimumWidth( 300 );

	catsView = new DesQCategoryView( this );
	appsView = new DesQApplicationsView( this );
	favsView = new DesQFavoritesView( this );

	catsView->setCategories( appsView->availableCategories() );

	connect(
		searchEdit, &QLineEdit::textChanged, this, [=]( QString filter ) {
			appsView->setFilter( filter );

			if ( filter.isEmpty() )
				catsView->setEnabled( true );

			else
				catsView->setDisabled( true );

			catsView->loadCategory( "All Applications" );
		}
	);

	connect( appsView, &DesQApplicationsView::menuReloaded, [=]() {
		catsView->setCategories( appsView->availableCategories() );
	} );

	connect( catsView, &DesQCategoryView::loadCategory, appsView, &DesQApplicationsView::loadCategory );
	connect( catsView, &DesQCategoryView::closeMenu, this, &DesQMenu::close );
	connect( appsView, &DesQApplicationsView::closeMenu, this, &DesQMenu::close );
	connect( favsView, &DesQFavoritesView::closeMenu, this, &DesQMenu::close );

	QGridLayout *viewLyt = new QGridLayout();
	viewLyt->setContentsMargins( 5, 15, 5, 15 );
	viewLyt->setSpacing( 0 );
	viewLyt->addWidget( catsView, 1, 0 );
	viewLyt->addWidget( appsView, 1, 1 );
	viewLyt->addWidget( favsView, 1, 2 );

	QFrame *menuFrame = new QFrame();
	menuFrame->setLayout( viewLyt );
	menuFrame->setObjectName( "frame" );
	menuFrame->setStyleSheet( "#frame{ border: none; border-top: 1px solid gray;; border-bottom: 1px solid gray; }" );

	QToolButton *powerBtn = new QToolButton();
	powerBtn->setIcon( QIcon::fromTheme( "system-shutdown" ) );
	powerBtn->setIconSize( QSize( 36, 36 ) );
	powerBtn->setAutoRaise( true );

	connect( powerBtn, &QToolButton::clicked, [=]() {
		close();

		LogoutDialog *dlg = new LogoutDialog();
		dlg->exec();
	} );

	QGridLayout *menuLyt = new QGridLayout();
	menuLyt->addWidget( searchEdit, 0, 0, 1, 2, Qt::AlignCenter );
	menuLyt->addWidget( menuFrame, 1, 0, 1, 2 );
	menuLyt->addWidget( powerBtn, 2, 0, Qt::AlignRight );

	setLayout( menuLyt );
};

void DesQMenu::mouseReleaseEvent( QMouseEvent *mEvent ) {

	QWidget::mouseReleaseEvent( mEvent );
	close();
};

void DesQMenu::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	QColor bgTint( palette().color( QPalette::Window ).darker() );
	bgTint.setAlpha( 210 );
	painter.fillRect( geometry(), bgTint );
	painter.end();

	QWidget::paintEvent( pEvent );
};

void DesQMenu::closeEvent( QCloseEvent *cEvent ) {

	emit closed();

	QWidget::closeEvent( cEvent );
};
